local AngryBossReminders = LibStub("AceAddon-3.0"):GetAddon("AngryBossReminders")
if not AngryBossReminders then
	print("Failed to load Boss Detection, can't find AngryBossReminders")
	return
end

local plugin = AngryBossReminders:NewModule("BossDetection", "AceEvent-3.0")
plugin.Callbacks = plugin.Callbacks or LibStub:GetLibrary("CallbackHandler-1.0"):New(plugin)

local encounterIDOverrides = { ["Fallen Protectors"] = 1598 }
local deadBosses = {}
ABRDeadBosses = deadBosses

function plugin:OnInitialize()
	self:RegisterEvent("UPDATE_INSTANCE_INFO", "CheckSavedInstanceData")
	self:RegisterEvent("ENCOUNTER_END", "EncounterEnd")
end

function plugin:OnEnable()
	RequestRaidInfo()
end

function plugin:IsBossKilled(encounterID)
	return deadBosses[encounterID] or false
end

function plugin:IsInLockedRaid(difficultyID)
	if not difficultyID then
		difficultyID = select(3, GetInstanceInfo())
	end
	return difficultyID and difficultyID >= 3 and difficultyID <= 6
end

function plugin:EncounterEnd(event, encounterID, encounterName, difficultyID, raidSize, endStatus)
	if endStatus == 1 and self:IsInLockedRaid(difficultyID) then
		deadBosses[ encounterID ] = true
		self.Callbacks:Fire("OnBossKill", encounterID)
	end
end

function plugin:GetEncounterId(instanceName, bossName)
	if encounterIDOverrides[ bossName ] then
		return encounterIDOverrides[ bossName ]
	end
	for _, instance in pairs(AngryBossReminders.Instances) do
		if instanceName == EJ_GetInstanceInfo(instance.journalID) then
			for _, boss in ipairs(instance.bosses) do
				if bossName == EJ_GetEncounterInfo( boss.journalID ) then
					return boss.encounterID
				end
			end
		end
	end
end

function plugin:CheckSavedInstanceData()
	wipe(deadBosses)
	for i = 1, GetNumSavedInstances() do
		local instanceName, raidID, reset, difficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName, numEncounters, encounterProgress = GetSavedInstanceInfo(i)
		if locked then
			local index = 1
			local bossName, _, dead = GetSavedInstanceEncounterInfo(i, index)
			while bossName do
				if dead then
					local encounterID = self:GetEncounterId(instanceName, bossName)
					if encounterID then deadBosses[ encounterID ] = true end
				end

				index = index + 1
				bossName, _, dead = GetSavedInstanceEncounterInfo(i, index)
		  end
		end
	end
end
